package taidegalleria;

import java.util.List;

/**
 * @author lainejee
 * @version 1.0 24.2.2025
 * |------------------------------------------------------------------------|
 * | Luokan nimi:   Tietokanta                          | Avustajat:        |
 * |-------------------------------------------------------------------------
 * | Vastuualueet:                                      |                   |
 * |                                                    | - Taiteilijat     |
 * |  - Huolehtii Taiteilijat ja Taideteokset -luokkien | - Taideteokset    |
 * |    välisestä yhteistyöstä ja välittää näitä        | - Taiteilija      |
 * |    tietoja pyydettäessä.                           | - Taideteos       |
 * |  - Lukee ja kirjoittaa tietokannan tiedostoon      |                   |
 * |    pyytämällä apua avustajiltaan.                  |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |                                                    |                   |
 * |-------------------------------------------------------------------------
 */

public class Tietokanta {

    private final Taiteilijat taiteilijat = new Taiteilijat();
    private final Taideteokset taideteokset = new Taideteokset();


    /**
     * Lisätään uusi taiteilija
     * @param taiteilija lisättävä taiteilija
     * @throws SailoException jos lisääminen ei onnistu
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Tietokanta lempparit = new Tietokanta();
     * Taiteilija hiroshige = new Taiteilija(), hopper = new Taiteilija();
     * hiroshige.rekisteroi(); hopper.rekisteroi();
     * lempparit.getTaiteilijoita() === 0;
     * lempparit.lisaa(hiroshige); lempparit.getTaiteilijoita() === 1;
     * lempparit.lisaa(hopper); lempparit.getTaiteilijoita() === 2;
     * lempparit.lisaa(hiroshige); lempparit.getTaiteilijoita() === 3;
     * lempparit.getTaiteilijoita() === 3;
     * lempparit.annaTaiteilija(0) === hiroshige;
     * lempparit.annaTaiteilija(1) === hopper;
     * lempparit.annaTaiteilija(2) === hiroshige;
     * lempparit.annaTaiteilija(3) === hiroshige; #THROWS IndexOutOfBoundsException
     * lempparit.lisaa(hiroshige); lempparit.getTaiteilijoita() === 4;
     * lempparit.lisaa(hiroshige); lempparit.getTaiteilijoita() === 5;
     * lempparit.lisaa(hiroshige); #THROWS SailoException
     * </pre>
     */
    public void lisaa(Taiteilija taiteilija) throws SailoException {
        taiteilijat.lisaa(taiteilija);
    }


    /**
     * Lisätään uusi taideteos tietokantaan.
     * @param teos lisättävä teos
     */
    public void lisaa(Taideteos teos) {
        taideteokset.lisaa(teos);
    }


    /**
     * @return taiteilijoiden lukumäärä
     */
    public int getTaiteilijoita() {
        return taiteilijat.getLkm();
    }


    /**
     * Anna tietokannan i:n taiteilija.
     * @param i monesko taiteilija (alkaa 0:sta)
     * @return taiteilija paikasta i
     */
    public Taiteilija annaTaiteilija(int i) {
        return taiteilijat.anna(i);
    }


    /**
     * Haetaan kaikki taiteilijan taideteokset.
     * @param taiteilija jonka taideteokset haetaan
     * @return tietorakenne jossa viitteet löydettyihin taideteoksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     *
     *  Tietokanta lempparit = new Tietokanta();
     *  Taiteilija hiroshige = new Taiteilija(), hopper = new Taiteilija(), monet = new Taiteilija();
     *  hiroshige.rekisteroi(); hopper.rekisteroi(); monet.rekisteroi();
     *  int id1 = hiroshige.getId();
     *  int id2 = hopper.getId();
     *  Taideteos t001 = new Taideteos(id1); lempparit.lisaa(t001);
     *  Taideteos t002 = new Taideteos(id1); lempparit.lisaa(t002);
     *  Taideteos t007 = new Taideteos(id2); lempparit.lisaa(t007);
     *  Taideteos t050 = new Taideteos(id2); lempparit.lisaa(t050);
     *  Taideteos t102 = new Taideteos(id2); lempparit.lisaa(t102);
     *
     *  List<Taideteos> loytyneet;
     *  loytyneet = lempparit.annaTaideteokset(monet);
     *  loytyneet.size() === 0;
     *  loytyneet = lempparit.annaTaideteokset(hiroshige);
     *  loytyneet.size() === 2;
     *  loytyneet.get(0) == t001 === true;
     *  loytyneet.get(1) == t002 === true;
     *  loytyneet = lempparit.annaTaideteokset(hopper);
     *  loytyneet.size() === 3;
     *  loytyneet.get(0) == t007 === true;
     * </pre>
     */
    public List<Taideteos> annaTaideteokset(Taiteilija taiteilija) {
        return taideteokset.annaTaideteokset(taiteilija.getId());
    }


    /**
     * Testataan tietokanta-luokkaa.
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Tietokanta suosikit = new Tietokanta();

        Taiteilija hiroshige = new Taiteilija();
        Taiteilija edward = new Taiteilija();

        hiroshige.rekisteroi();
        hiroshige.tayta();
        edward.rekisteroi();
        edward.tayta();

        try {
            suosikit.lisaa(hiroshige);
            suosikit.lisaa(edward);
        } catch (SailoException e) {
            // throw new RuntimeException(e);
            System.err.println(e.getMessage());
        }

        Taideteos teos1 = new Taideteos();
        teos1.vastaaTestiTaideteos(hiroshige.getId());
        Taideteos teos2 = new Taideteos();
        teos2.vastaaTestiTaideteos(hiroshige.getId());
        Taideteos teos3 = new Taideteos();
        teos3.vastaaTestiTaideteos(edward.getId());

        suosikit.lisaa(teos1);
        suosikit.lisaa(teos2);
        suosikit.lisaa(teos3);

        for (int i = 0; i < suosikit.getTaiteilijoita(); i++) {
            Taiteilija taiteilija = suosikit.annaTaiteilija(i);
            taiteilija.tulosta(System.out);

            List<Taideteos> taideteokset = suosikit.annaTaideteokset(taiteilija);

            for (Taideteos teos : taideteokset) {
                System.out.println(teos.getTaiteilijaTunnus() + " ");
                teos.tulosta(System.out);
            }
        }

    }

}
